@@ -1,20 +1,10 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
+from typing import Any
 from typing import Literal
 from typing import TypedDict
-from typing import Union
 
 import uuid
 
@@ -36,7 +26,6 @@ HAS_ACCOUNT_DEFAULT = _ACCOUNTDEFAULT()
 
 
 BoolSettings = Literal[
-    'additional_uri_schemes',
     'allow_open_file_uris',
     'always_english_wikipedia',
     'always_english_wiktionary',
@@ -51,6 +40,7 @@ BoolSettings = Literal[
     'dev_force_bookmark_2',
     'developer_modus',
     'enable_emoji_shortcodes',
+    'enable_keepassxc_integration',
     'enable_negative_priority',
     'enable_file_preview',
     'escape_key_closes',
@@ -75,15 +65,16 @@ BoolSettings = Literal[
     'preview_verify_https',
     'print_status_in_chats',
     'remote_control',
+    'show_main_menu',
     'save_main_window_position',
     'send_on_ctrl_enter',
-    'show_chatstate_in_banner',
     'show_help_start_chat',
     'show_in_taskbar',
     'show_notifications',
     'show_notifications_away',
     'show_only_chat_and_online',
     'show_send_message_button',
+    'show_voice_message_button',
     'show_status_msgs_in_roster',
     'show_subject_on_join',
     'show_transports_group',
@@ -126,8 +117,13 @@ IntSettings = Literal[
     'preview_size',
 ]
 
+FloatSettings = Literal[
+    'app_font_size',
+]
+
 StringSettings = Literal[
     'action_on_close',
+    'additional_uri_schemes',
     'audio_input_device',
     'audio_output_device',
     'autoaway_message',
@@ -148,6 +144,7 @@ StringSettings = Literal[
     'muclumbus_api_jid',
     'muclumbus_api_pref',
     'preview_leftclick_action',
+    'providers_list_url',
     'roster_theme',
     'search_engine',
     'show_main_window_on_startup',
@@ -159,10 +156,10 @@ StringSettings = Literal[
     'video_size',
 ]
 
-AllSettings = Literal[BoolSettings, IntSettings, StringSettings]
-AllSettingsT = Union[str, int, bool, list[str]]
+AllSettings = Literal[BoolSettings, IntSettings, FloatSettings, StringSettings]
+AllSettingsT = str | int | float | bool | list[str]
 
-APP_SETTINGS = {
+APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
     'additional_uri_schemes': '',
     'allow_open_file_uris': False,
     'always_english_wikipedia': False,
@@ -193,6 +190,7 @@ APP_SETTINGS = {
     'developer_modus': False,
     'dictionary_url': 'WIKTIONARY',
     'enable_emoji_shortcodes': True,
+    'enable_keepassxc_integration': False,
     'enable_negative_priority': False,
     'enable_file_preview': True,
     'escape_key_closes': False,
@@ -206,7 +204,7 @@ APP_SETTINGS = {
     'gc_sync_threshold_private_default': 0,
     'gc_sync_threshold_public_default': 1,
     'global_proxy': '',
-    'groupchat_roster_width': 210,
+    'groupchat_roster_width': 250,
     'hide_groupchat_occupants_list': False,
     'ignore_incoming_attention': False,
     'is_window_visible': True,
@@ -242,20 +240,22 @@ APP_SETTINGS = {
     'preview_max_file_size': 10485760,
     'preview_size': 300,
     'preview_verify_https': True,
+    'providers_list_url': 'https://data.xmpp.net/providers/v2/providers-B.json',
     'print_status_in_chats': False,
     'remote_control': False,
     'roster_theme': 'default',
     'save_main_window_position': True,
     'search_engine': 'https://duckduckgo.com/?q=%s',
     'send_on_ctrl_enter': False,
-    'show_chatstate_in_banner': True,
     'show_help_start_chat': True,
     'show_in_taskbar': True,
+    'show_main_menu': True,
     'show_main_window_on_startup': 'always',
     'show_notifications': True,
     'show_notifications_away': False,
     'show_only_chat_and_online': False,
     'show_send_message_button': False,
+    'show_voice_message_button': True,
     'show_status_msgs_in_roster': True,
     'show_subject_on_join': True,
     'show_transports_group': True,
@@ -273,9 +273,10 @@ APP_SETTINGS = {
     'use_keyring': True,
     'use_kib_mib': False,
     'use_libappindicator': True,
-    'use_speller': False,
+    'use_speller': True,
     'use_stun_server': False,
     'use_urgency_hint': True,
+    'app_font_size': 1.0,
     'video_framerate': '',
     'video_input_device': 'autovideosrc',
     'video_see_self': True,
@@ -290,6 +291,7 @@ BoolAccountSettings = Literal[
     'answer_receipts',
     'autoauth',
     'autoconnect',
+    'autojoin_sync',
     'client_cert_encrypted',
     'confirm_unencrypted_connection',
     'enable_gssapi',
@@ -312,13 +314,15 @@ BoolAccountSettings = Literal[
     'test_ft_proxies_on_startup',
     'use_custom_host',
     'use_ft_proxies',
-    'use_plain_connection'
+    'use_plain_connection',
+    'omemo_blind_trust'
 ]
 
 
 StringAccountSettings = Literal[
     'account_color',
     'account_label',
+    'address',
     'attached_gpg_keys',
     'avatar_sha',
     'client_cert',
@@ -328,13 +332,11 @@ StringAccountSettings = Literal[
     'file_transfer_proxies',
     'filetransfer_preference',
     'gc_send_chatstate_default',
-    'hostname',
     'http_auth',
     'keyid',
     'keyname',
     'last_status',
     'last_status_msg',
-    'name',
     'password',
     'proxy',
     'resource',
@@ -373,6 +375,7 @@ BoolGroupChatSettings = Literal[
 
 StringGroupChatSettings = Literal[
     'encryption',
+    'mute_until',
     'speller_language',
     'send_chatstate',
 ]
@@ -386,7 +389,7 @@ AllGroupChatSettings = Literal[BoolGroupChatSettings,
                                IntGroupChatSettings,
                                StringGroupChatSettings]
 
-AllGroupChatSettingsT = Union[str, int, bool]
+AllGroupChatSettingsT = str | int | bool
 
 
 BoolContactSettings = Literal[
@@ -395,6 +398,7 @@ BoolContactSettings = Literal[
 
 StringContactSettings = Literal[
     'encryption',
+    'mute_until',
     'speller_language',
     'send_chatstate',
 ]
@@ -402,7 +406,7 @@ StringContactSettings = Literal[
 AllContactSettings = Literal[BoolContactSettings,
                              StringContactSettings]
 
-AllContactSettingsT = Union[str, bool]
+AllContactSettingsT = str | bool
 
 
 ACCOUNT_SETTINGS = {
@@ -411,11 +415,13 @@ ACCOUNT_SETTINGS = {
         'account_label': '',
         'active': False,
         'adjust_priority_with_status': False,
+        'address': '',
         'anonymous_auth': False,
         'answer_receipts': True,
         'attached_gpg_keys': '',
         'autoauth': False,
         'autoconnect': True,
+        'autojoin_sync': True,
         'autopriority_away': 40,
         'autopriority_chat': 50,
         'autopriority_dnd': 20,
@@ -432,13 +438,13 @@ ACCOUNT_SETTINGS = {
         'default_workspace': '',
         'enable_gssapi': False,
         'enable_security_labels': False,
+        'encryption_default': '',
         'file_transfer_proxies': '',
         'filetransfer_preference': 'httpupload',
         'ft_send_local_ips': True,
         'gc_send_chatstate_default': 'composing_only',
         'gc_send_marker_private_default': True,
         'gc_send_marker_public_default': False,
-        'hostname': '',
         'http_auth': 'ask',
         'ignore_unknown_contacts': False,
         'is_zeroconf': False,
@@ -446,7 +452,6 @@ ACCOUNT_SETTINGS = {
         'keyname': '',
         'last_status': 'online',
         'last_status_msg': '',
-        'name': '',
         'password': '',
         'priority': 0,
         'proxy': '',
@@ -472,17 +477,20 @@ ACCOUNT_SETTINGS = {
         'zeroconf_first_name': '',
         'zeroconf_jabber_id': '',
         'zeroconf_last_name': '',
+        'omemo_blind_trust': True,
     },
 
     'contact': {
-        'encryption': '',
+        'encryption': HAS_ACCOUNT_DEFAULT,
+        'mute_until': '',
         'send_chatstate': HAS_ACCOUNT_DEFAULT,
         'send_marker': HAS_ACCOUNT_DEFAULT,
         'speller_language': '',
     },
 
     'group_chat': {
-        'encryption': '',
+        'encryption': HAS_ACCOUNT_DEFAULT,
+        'mute_until': '',
         'notify_on_all_messages': HAS_APP_DEFAULT,
         'print_join_left': HAS_APP_DEFAULT,
         'print_status': HAS_APP_DEFAULT,
@@ -511,7 +519,7 @@ class OpenChatSettingDetails(TypedDict):
 
 
 OpenChatsSettingT = list[OpenChatSettingDetails]
-AllWorkspaceSettingsT = Union[str, OpenChatsSettingT]
+AllWorkspaceSettingsT = str | OpenChatsSettingT
 
 
 class WorkspaceSettings(TypedDict):
@@ -582,6 +590,11 @@ PROXY_EXAMPLES = {
         'host': 'localhost',
         'port': 9050
     },
+    'Tor (Browser)': {
+        'type': 'socks5',
+        'host': 'localhost',
+        'port': 9150
+    },
 }
 
 
@@ -658,7 +671,6 @@ ADVANCED_SETTINGS = {
             'account in the Accounts window. BE CAREFUL, when you are logged '
             'in with a negative priority, you will NOT receive any message '
             'from your server.'),
-        'escape_key_closes': _('Close a chat by pressing the Escape key'),
         'file_transfers_port': '',
         'ft_add_hosts_to_send': _(
             'List of send hosts (comma separated) in '
@@ -686,14 +698,14 @@ ADVANCED_SETTINGS = {
         'notify_on_all_muc_messages': '',
         'plugins_repository_enabled': _(
             'If enabled, Gajim offers to download plugins hosted on gajim.org'),
+        'providers_list_url': _(
+            'Endpoint for retrieving a list of providers for sign up'),
         'save_main_window_position': _(
             'If enabled, Gajim will save the main window position when hiding '
             'it, and restore it when showing the window again.'),
         'search_engine': '',
         'send_on_ctrl_enter': _(
             'Send message on Ctrl+Enter and make a new line with Enter.'),
-        'show_chatstate_in_banner': _(
-            'Show chat state (e.g. "is typing…") next to your contact’s name'),
         'stun_server': _('STUN server to use when using Jingle'),
         'time_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
         'trayicon_notification_on_events': _(
