@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -19,32 +9,31 @@ from typing import cast
 from typing import Literal
 
 import logging
-import time
 from collections.abc import Generator
 from datetime import datetime
 from datetime import timedelta
 
 from gi.repository import Gdk
 from gi.repository import Gio
+from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
 from nbxmpp.errors import StanzaError
-from nbxmpp.modules.security_labels import Displaymarking
 from nbxmpp.protocol import JID
-from nbxmpp.structs import CommonError
 from nbxmpp.structs import MucSubject
 
 from gajim.common import app
 from gajim.common import events
 from gajim.common import types
-from gajim.common.helpers import AdditionalDataDict
-from gajim.common.helpers import get_start_of_day
+from gajim.common.const import Direction
 from gajim.common.helpers import to_user_string
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.httpupload import HTTPFileTransfer
-from gajim.common.storage.archive import ConversationRow
+from gajim.common.storage.archive.const import ChatDirection
+from gajim.common.storage.archive.models import Message
 from gajim.common.types import ChatContactT
+from gajim.common.util.datetime import get_start_of_day
 
 from gajim.gtk.conversation.rows.base import BaseRow
 from gajim.gtk.conversation.rows.call import CallRow
@@ -61,6 +50,7 @@ from gajim.gtk.conversation.rows.muc_subject import MUCSubject
 from gajim.gtk.conversation.rows.read_marker import ReadMarkerRow
 from gajim.gtk.conversation.rows.scroll_hint import ScrollHintRow
 from gajim.gtk.conversation.rows.user_status import UserStatus
+from gajim.gtk.conversation.rows.widgets import MessageRowActions
 
 log = logging.getLogger('gajim.gtk.conversation_view')
 
@@ -80,7 +70,7 @@ class ConversationView(Gtk.ScrolledWindow):
         )
     }
 
-    def __init__(self) -> None:
+    def __init__(self, message_row_actions: MessageRowActions) -> None:
         Gtk.ScrolledWindow.__init__(self)
 
         self.set_overlay_scrolling(False)
@@ -101,13 +91,13 @@ class ConversationView(Gtk.ScrolledWindow):
         # horizontally under certain conditions (applies to GroupchatControl)
         self.get_hscrollbar().hide()
 
-        self._list_box = Gtk.ListBox()
-        self._list_box.set_selection_mode(Gtk.SelectionMode.NONE)
-        self._list_box.set_sort_func(self._sort_func)
+        self._message_row_actions = message_row_actions
 
         self._contact: ChatContactT | None = None
         self._client = None
 
+        self._list_box = Gtk.ListBox()
+
         # Keeps track of the number of rows shown in ConversationView
         self._row_count: int = 0
         self._max_row_count: int = 100
@@ -115,8 +105,8 @@ class ConversationView(Gtk.ScrolledWindow):
         # Keeps track of date rows we have added to the list
         self._active_date_rows: set[datetime] = set()
 
-        # message_id -> row mapping
         self._message_id_row_map: dict[str, MessageRow] = {}
+        self._stanza_id_row_map: dict[str, MessageRow] = {}
 
         self._read_marker_row = None
         self._scroll_hint_row = None
@@ -146,7 +136,7 @@ class ConversationView(Gtk.ScrolledWindow):
         for row in cast(list[BaseRow], self._list_box.get_selected_rows()):
             if isinstance(row, MessageRow):
                 timestamp_formatted = row.timestamp.strftime(format_string)
-                selection_text += (f'{timestamp_formatted} - {row.name}: '
+                selection_text += (f'{timestamp_formatted} - {row.name}:\n'
                                    f'{row.get_text()}\n')
 
         clip = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
@@ -155,13 +145,13 @@ class ConversationView(Gtk.ScrolledWindow):
         self.disable_row_selection()
 
     def enable_row_selection(self,
-                             log_line_id: int | None
+                             pk: int | None
                              ) -> None:
 
         self._list_box.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
 
-        if log_line_id is not None:
-            row = self.get_row_by_log_line_id(log_line_id)
+        if pk is not None:
+            row = self.get_row_by_pk(pk)
             self._list_box.select_row(row)
 
         for row in self.iter_rows():
@@ -254,6 +244,18 @@ class ConversationView(Gtk.ScrolledWindow):
             log.debug('emit %s, %s', signal_name, args)
             self.emit(signal_name, *args)
 
+    def _reset_list_box(self) -> None:
+        self._list_box.destroy()
+        self._list_box = Gtk.ListBox()
+        self._list_box.set_selection_mode(Gtk.SelectionMode.NONE)
+        self._list_box.set_sort_func(self._sort_func)
+        self._list_box.show()
+
+        current_child = self.get_child()
+        assert current_child is not None
+        current_child.destroy()
+        self.add(self._list_box)
+
     def _reset(self) -> None:
         self._current_upper = 0
         self._autoscroll = True
@@ -263,12 +265,12 @@ class ConversationView(Gtk.ScrolledWindow):
         self._requesting = None
         self.set_history_complete(True, False)
 
-        for row in self._list_box.get_children():
-            row.destroy()
+        self._reset_list_box()
 
         self._row_count = 0
         self._active_date_rows = set()
         self._message_id_row_map = {}
+        self._stanza_id_row_map = {}
         self._read_marker_row = None
         self._scroll_hint_row = None
 
@@ -374,15 +376,33 @@ class ConversationView(Gtk.ScrolledWindow):
         assert row is not None
         return cast(BaseRow, row)
 
-    def get_first_message_row(self) -> MessageRow | None:
+    def get_first_row(self
+    ) -> MessageRow | CallRow | FileTransferJingleRow | None:
+        for row in self._list_box.get_children():
+            if isinstance(row, MessageRow | CallRow | FileTransferJingleRow):
+                return row
+        return None
+
+    def get_last_row(
+        self
+    ) -> MessageRow | CallRow | FileTransferJingleRow | None:
+        children = reversed(self._list_box.get_children())
+        for row in children:
+            if isinstance(row, MessageRow | CallRow | FileTransferJingleRow):
+                return row
+        return None
+
+    def get_first_message_row(self
+    ) -> MessageRow | None:
         for row in self._list_box.get_children():
             if isinstance(row, MessageRow):
                 return row
         return None
 
-    def get_last_message_row(self) -> MessageRow | None:
-        children = self._list_box.get_children()
-        children.reverse()
+    def get_last_message_row(
+        self
+    ) -> MessageRow | None:
+        children = reversed(self._list_box.get_children())
         for row in children:
             if isinstance(row, MessageRow):
                 return row
@@ -448,7 +468,7 @@ class ConversationView(Gtk.ScrolledWindow):
 
     def add_info_message(self,
                          text: str,
-                         timestamp: float | None = None
+                         timestamp: datetime | None = None
                          ) -> None:
 
         message = InfoMessage(self.contact.account, text, timestamp)
@@ -463,7 +483,7 @@ class ConversationView(Gtk.ScrolledWindow):
         event: (events.FileRequestReceivedEvent |
                 events.FileRequestSent |
                 None) = None,
-        db_message: ConversationRow | None = None
+        message: Message | None = None
     ) -> None:
 
         assert isinstance(self._contact, BareContact)
@@ -471,7 +491,7 @@ class ConversationView(Gtk.ScrolledWindow):
             self._contact.account,
             self._contact,
             event=event,
-            db_message=db_message)
+            message=message)
         self._insert_message(jingle_transfer_row)
 
     def add_encryption_info(self, event: events.EncryptionInfo) -> None:
@@ -480,14 +500,14 @@ class ConversationView(Gtk.ScrolledWindow):
 
     def add_call_message(self,
                          event: events.JingleRequestReceived | None = None,
-                         db_message: ConversationRow | None = None
+                         message: Message | None = None
                          ) -> None:
         assert isinstance(self._contact, BareContact)
         call_row = CallRow(
             self._contact.account,
             self._contact,
             event=event,
-            db_message=db_message)
+            message=message)
         self._insert_message(call_row)
 
     def add_command_output(self, text: str, is_error: bool) -> None:
@@ -495,47 +515,34 @@ class ConversationView(Gtk.ScrolledWindow):
             self.contact.account, text, is_error)
         self._insert_message(command_output_row)
 
-    def add_message(self,
-                    text: str,
-                    kind: str,
-                    name: str,
-                    timestamp: float,
-                    log_line_id: int | None = None,
-                    message_id: str | None = None,
-                    stanza_id: str | None = None,
-                    display_marking: Displaymarking | None = None,
-                    additional_data: AdditionalDataDict | None = None,
-                    marker: str | None = None,
-                    error: CommonError | StanzaError | None = None
-                    ) -> None:
+    def add_message_from_db(self, message: Message) -> None:
+        message_row = MessageRow.from_db_row(self.contact, message)
+        message_row.connect(
+            'state-flags-changed', self._on_message_row_state_flags_changed)
+        message_id = message.id
+        stanza_id = message.stanza_id
 
-        if not timestamp:
-            timestamp = time.time()
-
-        message_row = MessageRow(
-            self.contact.account,
-            self.contact,
-            message_id,
-            stanza_id,
-            timestamp,
-            kind,
-            name,
-            text,
-            additional_data=additional_data,
-            display_marking=display_marking,
-            marker=marker,
-            error=error,
-            log_line_id=log_line_id)
+        if stanza_id is not None:
+            self._stanza_id_row_map[stanza_id] = message_row
 
         if message_id is not None:
             self._message_id_row_map[message_id] = message_row
 
-        if kind == 'incoming':
+        if message.corrections:
+            # Store the same MessageRow object also with the message id
+            # of the last correction, because we need it for XEP-0184 Receipts
+            # which does not reference the original message id.
+            corr_message_id = message.get_last_correction().id
+            if corr_message_id is not None:
+                self._message_id_row_map[corr_message_id] = message_row
+
+        if message.direction == ChatDirection.INCOMING:
             assert self._read_marker_row is not None
             self._read_marker_row.set_last_incoming_timestamp(
                 message_row.timestamp)
-        if (marker is not None and marker == 'displayed' and
-                message_id is not None):
+
+        if message.markers:
+            assert message_id is not None
             self.set_read_marker(message_id)
 
         self._insert_message(message_row)
@@ -546,12 +553,12 @@ class ConversationView(Gtk.ScrolledWindow):
         self._check_for_merge(message)
         assert self._read_marker_row is not None
 
-        if message.kind == 'incoming':
+        if message.direction == ChatDirection.INCOMING:
             if message.timestamp > self._read_marker_row.timestamp:
                 self._read_marker_row.hide()
 
     def _add_date_row(self, timestamp: datetime) -> None:
-        start_of_day = get_start_of_day(timestamp)
+        start_of_day = get_start_of_day(timestamp.astimezone())
         if start_of_day in self._active_date_rows:
             return
 
@@ -579,8 +586,7 @@ class ConversationView(Gtk.ScrolledWindow):
         if ancestor is None:
             self._update_descendants(message)
         else:
-            if message.is_mergeable(ancestor):
-                message.set_merged(True)
+            message.set_merged(message.is_mergeable(ancestor))
 
     def _find_ancestor(self, message: MessageRow) -> MessageRow | None:
         index = message.get_index()
@@ -621,56 +627,33 @@ class ConversationView(Gtk.ScrolledWindow):
             row.set_merged(merge)
             return
 
-    def reduce_message_count(self, before: bool) -> bool:
-        success = False
-        row_count = len(self._list_box.get_children())
-        while row_count > self._max_row_count:
-            if before:
-                if self._reduce_messages_before():
-                    row_count -= 1
-                    success = True
-            else:
-                self._reduce_messages_after()
-                row_count -= 1
-                success = True
+    def _on_message_row_state_flags_changed(
+        self,
+        row: MessageRow,
+        previous_flags: Gtk.StateFlags
+    ) -> None:
+        current_flags = self.get_state_flags()
 
-        return success
+        if (current_flags & Gtk.StateFlags.BACKDROP or
+                previous_flags & Gtk.StateFlags.BACKDROP):
+            # Don't react to backdrop changes and focus changes when in background
+            return
 
-    def _reduce_messages_before(self) -> bool:
-        success = False
+        if previous_flags & Gtk.StateFlags.PRELIGHT:
+            self._message_row_actions.hide_actions()
+        else:
+            coords = row.translate_coordinates(self, 0, 0)
+            if coords is None:
+                return
+            _x_coord, y_coord = coords
+            self._message_row_actions.update(y_coord, row)
 
-        # We want to keep relevant DateRows when removing rows
-        row1 = self._get_row_at_index(2)
-        row2 = self._get_row_at_index(3)
-
-        if row1.type == 'date' and row2.type == 'date':
-            # First two rows are date rows,
-            # it’s safe to remove the fist row
-            row1.destroy()
-            success = True
-
-        if row1.type == 'date' and row2.type != 'date':
-            # First one is a date row, keep it and
-            # remove the second row instead
-            row2.destroy()
-            success = True
-
-        if row1.type != 'date':
-            # Not a date row, safe to remove
-            row1.destroy()
-            success = True
-
-        return success
-
-    def _reduce_messages_after(self) -> None:
-        row = self._get_row_at_index(len(self._list_box.get_children()) - 1)
-        row.destroy()
-
-    def remove_message(self, log_line_id: int) -> None:
-        row = self.get_row_by_log_line_id(log_line_id)
+    def remove_message(self, pk: int) -> None:
+        row = self.get_row_by_pk(pk)
         if row is None:
             return
 
+        self._remove_from_maps(row)
         index = row.get_index()
         row.destroy()
         decendant_row = self._list_box.get_row_at_index(index)
@@ -680,47 +663,125 @@ class ConversationView(Gtk.ScrolledWindow):
             # unset merged state.
             decendant_row.set_merged(False)
 
-    def scroll_to_message_and_highlight(self, log_line_id: int) -> None:
+    def _remove_from_maps(self, row: MessageRow) -> None:
+        for key, val in dict(self._message_id_row_map).items():
+            if val is row:
+                del self._message_id_row_map[key]
+
+        for key, val in dict(self._stanza_id_row_map).items():
+            if val is row:
+                del self._stanza_id_row_map[key]
+
+    def acknowledge_message(self, event: events.MessageAcknowledged) -> None:
+        row = self.get_row_by_pk(event.pk)
+        if row is None:
+            return
+
+        if event.stanza_id is not None:
+            self._stanza_id_row_map[event.stanza_id] = row
+        row.set_acknowledged(event.stanza_id)
+        self._check_for_merge(row)
+
+    def scroll_to_message_and_highlight(self, pk: int) -> None:
         highlight_row = None
         for row in cast(list[BaseRow], self._list_box.get_children()):
-            row.get_style_context().remove_class(
-                'conversation-search-highlight')
-            if row.log_line_id == log_line_id:
+            if row.pk == pk:
                 highlight_row = row
+                break
 
-        if highlight_row is not None:
-            highlight_row.get_style_context().add_class(
-                'conversation-search-highlight')
-            # This scrolls the ListBox to the highlighted row
-            highlight_row.grab_focus()
+        if highlight_row is None:
+            return
+
+        # Scroll ListBox to row and highlight it
+        coordinates = highlight_row.translate_coordinates(self._list_box, 0, 0)
+        if coordinates is None:
+            return
+
+        _x_coord, y_coord = coordinates
+        _minimum_height, natural_height = highlight_row.get_preferred_height()
+        adjustment = self._list_box.get_adjustment()
+        adjustment.set_value(
+            y_coord - (adjustment.get_page_size() - natural_height) / 2)
+
+        highlight_row.get_style_context().remove_class(
+            'conversation-row-highlight')
+        highlight_row.get_style_context().add_class(
+            'conversation-row-highlight')
+
+        GLib.timeout_add(1500, self._remove_highligh_class, highlight_row)
+
+    def _remove_highligh_class(self, highlight_row: BaseRow) -> None:
+        highlight_row.get_style_context().remove_class(
+            'conversation-row-highlight')
 
     def scroll_to_end(self) -> None:
         adj = self.get_vadjustment()
         adj.set_value(adj.get_upper() - adj.get_page_size())
 
-    def _get_row_by_message_id(self, id_: str) -> MessageRow | None:
-        return self._message_id_row_map.get(id_)
+    def _get_row_by_message_id(self, message_id: str) -> MessageRow | None:
+        return self._message_id_row_map.get(message_id)
 
-    def get_row_by_log_line_id(self, log_line_id: int) -> MessageRow | None:
+    def _get_row_by_stanza_id(self, stanza_id: str) -> MessageRow | None:
+        return self._stanza_id_row_map.get(stanza_id)
+
+    def _get_message_row_by_direction(
+        self,
+        pk: int,
+        direction: Direction | None = None
+    ) -> MessageRow | None:
+
+        row = self.get_row_by_pk(pk)
+        if row is None:
+            return None
+
+        if direction is None:
+            return row
+
+        index = row.get_index()
+        while True:
+            if direction == Direction.PREV:
+                index -= 1
+            else:
+                index += 1
+
+            row = self._list_box.get_row_at_index(index)
+            if row is None:
+                return None
+
+            if isinstance(row, MessageRow):
+                return row
+
+    def get_prev_message_row(
+        self,
+        pk: int | None
+    ) -> MessageRow | None:
+        if pk is None:
+            return self.get_last_message_row()
+        return self._get_message_row_by_direction(
+            pk, direction=Direction.PREV)
+
+    def get_next_message_row(
+        self,
+        pk: int | None
+    ) -> MessageRow | None:
+        if pk is None:
+            return None
+        return self._get_message_row_by_direction(
+            pk, direction=Direction.NEXT)
+
+    def get_row_by_pk(self, pk: int) -> MessageRow | None:
         for row in cast(list[BaseRow], self._list_box.get_children()):
             if not isinstance(row, MessageRow):
                 continue
-            if row.log_line_id == log_line_id:
+            if pk in {row.pk, row.orig_pk}:
                 return row
-        return None
 
-    def get_row_by_stanza_id(self, stanza_id: str) -> MessageRow | None:
-        for row in cast(list[BaseRow], self._list_box.get_children()):
-            if not isinstance(row, MessageRow):
-                continue
-            if row.stanza_id == stanza_id:
-                return row
         return None
 
     def iter_rows(self) -> Generator[BaseRow, None, None]:
         yield from cast(list[BaseRow], self._list_box.get_children())
 
-    def remove_rows_by_type(self, row_type: str) -> None:
+    def _remove_rows_by_type(self, row_type: str) -> None:
         for row in self.iter_rows():
             if row.type == row_type:
                 row.destroy()
@@ -735,7 +796,6 @@ class ConversationView(Gtk.ScrolledWindow):
         if row is None:
             return
 
-        row.set_displayed()
         assert self._read_marker_row is not None
         timestamp = row.timestamp + timedelta(microseconds=1)
         if self._read_marker_row.timestamp > timestamp:
@@ -743,36 +803,53 @@ class ConversationView(Gtk.ScrolledWindow):
 
         self._read_marker_row.set_timestamp(timestamp)
 
-    def update_avatars(self) -> None:
-        for row in cast(list[BaseRow], self._list_box.get_children()):
-            if isinstance(row, MessageRow):
-                row.update_avatar()
+    def correct_message(self, event: events.MessageCorrected) -> None:
+        message_row = self._get_row_by_message_id(event.correction_id)
+        if message_row is None:
+            return
 
-    def correct_message(self,
-                        correct_id: str,
-                        text: str,
-                        nickname: str | None
-                        ) -> None:
+        # Store the message row also with the correction message id
+        # because e.g. message receipts reference this id.
+        if event.message.id is not None:
+            self._message_id_row_map[event.message.id] = message_row
 
-        message_row = self._get_row_by_message_id(correct_id)
+        message_row.refresh()
+
+        assert self._read_marker_row is not None
+        timestamp = message_row.timestamp + timedelta(microseconds=1)
+        if self._read_marker_row.timestamp == timestamp:
+            # This exact message has been marked as read
+            # -> set read marker to before this message
+            self._read_marker_row.set_timestamp(
+                message_row.timestamp - timedelta(microseconds=1), force=True)
+
+    def show_message_moderation(self, stanza_id: str, text: str) -> None:
+        message_row = self._get_row_by_stanza_id(stanza_id)
         if message_row is not None:
-            message_row.set_correction(text, nickname)
-            message_row.set_merged(False)
+            message_row.set_moderated(text)
+
+    def update_message_reactions(self, reaction_id: str) -> None:
+        if isinstance(self._contact, GroupchatContact):
+            message_row = self._get_row_by_stanza_id(reaction_id)
+        else:
+            message_row = self._get_row_by_message_id(reaction_id)
 
-    def show_message_retraction(self, stanza_id: str, text: str) -> None:
-        message_row = self.get_row_by_stanza_id(stanza_id)
         if message_row is not None:
-            message_row.set_retracted(text)
+            message_row.update_reactions()
 
-    def show_receipt(self, id_: str) -> None:
+    def set_receipt(self, id_: str) -> None:
         message_row = self._get_row_by_message_id(id_)
-        if message_row is not None:
-            message_row.set_receipt()
+        if message_row is None:
+            return
+
+        if message_row.last_message_id == id_:
+            message_row.set_receipt(True)
+            self._check_for_merge(message_row)
 
     def show_error(self, id_: str, error: StanzaError) -> None:
         message_row = self._get_row_by_message_id(id_)
         if message_row is not None:
-            message_row.set_error(to_user_string(error))
+            message_row.show_error(to_user_string(error))
             message_row.set_merged(False)
 
     def _on_contact_setting_changed(self,
@@ -784,14 +861,10 @@ class ConversationView(Gtk.ScrolledWindow):
         if setting == 'print_join_left':
             if value:
                 return
-            self.remove_rows_by_type('muc-user-joined')
-            self.remove_rows_by_type('muc-user-left')
+            self._remove_rows_by_type('muc-user-joined')
+            self._remove_rows_by_type('muc-user-left')
 
         if setting == 'print_status':
             if value:
                 return
-            self.remove_rows_by_type('muc-user-status')
-
-    def remove(self, widget: Gtk.Widget) -> None:
-        self._list_box.remove(widget)
-        widget.destroy()
+            self._remove_rows_by_type('muc-user-status')
