@@ -1,22 +1,11 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import cast
 from typing import Literal
-from typing import Optional
 from typing import overload
 
 import logging
@@ -35,8 +24,8 @@ from gajim.common import ged
 from gajim.common.const import ArchiveState
 from gajim.common.events import ArchivingIntervalFinished
 from gajim.common.events import RawMamMessageReceived
-from gajim.common.helpers import event_filter
 from gajim.common.i18n import _
+from gajim.common.util.decorators import event_filter
 
 from gajim.gtk.assistant import Assistant
 from gajim.gtk.assistant import Page
@@ -56,25 +45,23 @@ class HistorySyncAssistant(Assistant, EventHelper):
         self.account = account
         self._client = app.get_client(account)
 
-        self._timedelta: Optional[timedelta] = None
+        self._timedelta: timedelta | None = None
         self._now = datetime.now(timezone.utc)
-        self._query_id: Optional[str] = None
-        self._start: Optional[datetime] = None
-        self._end: Optional[datetime] = None
+        self._query_id: str | None = None
+        self._start: datetime = ArchiveState.ALL
+        self._end: datetime | None = None
 
-        mam_start = ArchiveState.NEVER
-        archive = app.storage.archive.get_archive_infos(
-            self._client.get_own_jid().bare)
-        if archive is not None and archive.oldest_mam_timestamp is not None:
-            mam_start = int(float(archive.oldest_mam_timestamp))
+        mam_start = None
+        archive = app.storage.archive.get_mam_archive_state(
+            account, self._client.get_own_jid().new_as_bare())
 
-        if mam_start == ArchiveState.NEVER:
+        if archive is not None and archive.from_stanza_ts is not None:
+            mam_start = archive.from_stanza_ts
+
+        if mam_start is None:
             self._current_start = self._now
-        elif mam_start == ArchiveState.ALL:
-            self._current_start = datetime.fromtimestamp(0, timezone.utc)
         else:
-            self._current_start = datetime.fromtimestamp(mam_start,
-                                                         timezone.utc)
+            self._current_start = mam_start
 
         self.add_button('synchronize', _('Synchronize'), 'suggested-action')
         self.add_button('close', _('Close'))
@@ -94,7 +81,6 @@ class HistorySyncAssistant(Assistant, EventHelper):
 
         # pylint: disable=line-too-long
         self.register_events([
-            ('archiving-count-received', ged.GUI1, self._received_count),
             ('archiving-interval-finished', ged.GUI1, self._received_finished),
             ('raw-mam-message-received',
              ged.PRECORE,
@@ -104,7 +90,7 @@ class HistorySyncAssistant(Assistant, EventHelper):
 
         if mam_start == ArchiveState.ALL:
             success_page.set_text(
-                _('Gajim is fully synchronised with the archive.'))
+                _('Gajim is fully synchronized with the archive.'))
             self.show_page('success')
 
         self.show_all()
@@ -158,6 +144,7 @@ class HistorySyncAssistant(Assistant, EventHelper):
         log.info('Start: %s', self._start)
         log.info('End: %s', self._end)
 
+        # Archive count query
         self._client.get_module('MAM').make_query(
             self._client.get_own_jid().bare,
             start=self._start,
@@ -174,6 +161,8 @@ class HistorySyncAssistant(Assistant, EventHelper):
         if result.rsm.count is not None:
             self.get_page('progress').set_count(int(result.rsm.count))
         mam_module = self._client.get_module('MAM')
+        assert self._start is not None
+        assert self._end is not None
         self._query_id = mam_module.request_archive_interval(
             self._start, self._end)
 
@@ -210,7 +199,7 @@ class SelectTime(Page):
         self.title = _('Synchronize Chat History')
 
         self.complete = False
-        self._timedelta: Optional[timedelta] = None
+        self._timedelta: timedelta | None = None
 
         heading = Gtk.Label()
         heading.get_style_context().add_class('large-header')
@@ -221,7 +210,7 @@ class SelectTime(Page):
         heading.set_text(_('Synchronize Chat History'))
 
         label = Gtk.Label(
-            label=_('How far back should the chat history be synchronised?'))
+            label=_('How far back should the chat history be synchronized?'))
         label.set_halign(Gtk.Align.CENTER)
         label.set_line_wrap(True)
         label.set_max_width_chars(40)
@@ -251,10 +240,10 @@ class SelectTime(Page):
 
     def _on_row_selected(self, _listbox: Gtk.ListBox, row: TimeOption) -> None:
         self._timedelta = row.get_timedelta()
-        self.complete = row is not None
+        self.complete = True
         self.update_page_complete()
 
-    def get_timedelta(self) -> Optional[timedelta]:
+    def get_timedelta(self) -> timedelta | None:
         return self._timedelta
 
     def get_default_button(self) -> str:
@@ -307,12 +296,12 @@ class Progress(Page):
 
 
 class TimeOption(Gtk.ListBoxRow):
-    def __init__(self, text: str, months: Optional[timedelta] = None) -> None:
+    def __init__(self, text: str, months: timedelta | None = None) -> None:
         Gtk.ListBoxRow.__init__(self)
         label = Gtk.Label(label=text)
         self.add(label)
 
         self._timedelta = months
 
-    def get_timedelta(self) -> Optional[timedelta]:
+    def get_timedelta(self) -> timedelta | None:
         return self._timedelta
