@@ -1,62 +1,47 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
+
 from typing import Any
-from typing import Optional
 from typing import cast
-from typing import Type
 
 import logging
 import sys
 
-from gi.repository import Gtk
 from gi.repository import Gdk
+from gi.repository import Gtk
 
 from gajim.common import app
 from gajim.common import configpaths
-from gajim.common import helpers
 from gajim.common.const import THRESHOLD_OPTIONS
-from gajim.common.i18n import _
-from gajim.common.helpers import open_file
-from gajim.common.multimedia_helpers import AudioInputManager
-from gajim.common.multimedia_helpers import AudioOutputManager
-from gajim.common.multimedia_helpers import VideoInputManager
 from gajim.common.events import StyleChanged
 from gajim.common.events import ThemeUpdate
+from gajim.common.i18n import _
 from gajim.common.setting_values import BoolSettings
+from gajim.common.util.av import AudioInputManager
+from gajim.common.util.av import AudioOutputManager
+from gajim.common.util.av import VideoInputManager
+from gajim.common.util.uri import open_directory
+from gajim.common.util.version import package_version
 
-from .const import Setting
-from .const import SettingKind
-from .const import SettingType
-from .dialogs import DialogButton
-from .dialogs import ConfirmationDialog
-from .emoji_chooser import emoji_chooser
-from .settings import PopoverSetting
-from .settings import SettingsBox
-from .settings import SettingsDialog
-from .sidebar_switcher import SideBarSwitcher
-from .video_preview import VideoPreview
-from .preview import PREVIEW_CLICK_ACTIONS
-from .util import open_window
-from .util import get_app_window
-from .builder import get_builder
+from gajim.gtk.builder import get_builder
+from gajim.gtk.const import Setting
+from gajim.gtk.const import SettingKind
+from gajim.gtk.const import SettingType
+from gajim.gtk.dialogs import ConfirmationDialog
+from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.preview import PREVIEW_ACTIONS
+from gajim.gtk.settings import PopoverSetting
+from gajim.gtk.settings import SettingsBox
+from gajim.gtk.settings import SettingsDialog
+from gajim.gtk.sidebar_switcher import SideBarSwitcher
+from gajim.gtk.util import get_app_window
+from gajim.gtk.util import open_window
+from gajim.gtk.video_preview import VideoPreview
 
-# if app.is_installed('GSPELL'):
-#     from gi.repository import Gspell  # pylint: disable=ungrouped-imports
-
-log = logging.getLogger('gajim.gui.preferences')
+log = logging.getLogger('gajim.gtk.preferences')
 
 
 class Preferences(Gtk.ApplicationWindow):
@@ -72,7 +57,7 @@ class Preferences(Gtk.ApplicationWindow):
 
         self._ui = get_builder('preferences.ui')
 
-        self._video_preview: Optional[VideoPreview] = None
+        self._video_preview: VideoPreview | None = None
         self._prefs: dict[str, PreferenceBox] = {}
 
         side_bar_switcher = SideBarSwitcher()
@@ -81,9 +66,7 @@ class Preferences(Gtk.ApplicationWindow):
 
         self.add(self._ui.grid)
 
-        self._check_emoji_theme()
-
-        prefs: list[tuple[str, Type[PreferenceBox]]] = [
+        prefs: list[tuple[str, type[PreferenceBox]]] = [
             ('window_behaviour', WindowBehaviour),
             ('plugins', Plugins),
             ('general', General),
@@ -95,7 +78,6 @@ class Preferences(Gtk.ApplicationWindow):
             ('status_message', StatusMessage),
             ('automatic_status', AutomaticStatus),
             ('themes', Themes),
-            ('emoji', Emoji),
             ('server', Server),
             ('audio', Audio),
             ('video', Video),
@@ -117,30 +99,26 @@ class Preferences(Gtk.ApplicationWindow):
         self._ui.connect_signals(self)
 
         self.show_all()
-        if sys.platform != 'darwin':
-            # TODO: Remove if colored emoji rendering works well on
-            # Windows and MacOS
-            self._ui.emoji.hide()
 
     def get_ui(self):
         return self._ui
 
-    def _add_prefs(self, prefs: list[tuple[str, Type[PreferenceBox]]]):
+    def _add_prefs(self, prefs: list[tuple[str, type[PreferenceBox]]]):
         for ui_name, klass in prefs:
             pref_box = getattr(self._ui, ui_name)
-            pref = klass(self)
+            pref = klass(self)  # pyright: ignore
             pref_box.add(pref)
             self._prefs[ui_name] = pref
 
     def _add_video_preview(self) -> None:
         self._video_preview = VideoPreview()
-        self._ui.video.add(self._video_preview.widget)
+        self._ui.video.add(self._video_preview)
 
     def _on_key_press(self, _widget: Gtk.Widget, event: Gdk.EventKey) -> None:
         if event.keyval == Gdk.KEY_Escape:
             self.destroy()
 
-    def get_video_preview(self) -> Optional[VideoPreview]:
+    def get_video_preview(self) -> VideoPreview | None:
         return self._video_preview
 
     @staticmethod
@@ -157,14 +135,6 @@ class Preferences(Gtk.ApplicationWindow):
         miscellaneous = cast(Miscellaneous, self._prefs['miscellaneous'])
         miscellaneous.update_proxy_list()
 
-    @staticmethod
-    def _check_emoji_theme() -> None:
-        # Ensure selected emoji theme is valid
-        emoji_themes = helpers.get_available_emoticon_themes()
-        settings_theme = app.settings.get('emoticons_theme')
-        if settings_theme not in emoji_themes:
-            app.settings.set('emoticons_theme', 'font')
-
     def _on_destroy(self, _widget: Gtk.Widget) -> None:
         self._prefs.clear()
         app.check_finalize(self)
@@ -173,7 +143,7 @@ class Preferences(Gtk.ApplicationWindow):
 class PreferenceBox(SettingsBox):
     def __init__(self, settings: list[Setting]) -> None:
         SettingsBox.__init__(self, None)
-        self.get_style_context().add_class('settings-border')
+        self.get_style_context().add_class('border')
         self.set_selection_mode(Gtk.SelectionMode.NONE)
         self.set_vexpand(False)
         self.set_valign(Gtk.Align.END)
@@ -279,6 +249,11 @@ class General(PreferenceBox):
                     SettingType.CONFIG,
                     'show_send_message_button'),
 
+            Setting(SettingKind.SWITCH,
+                    _('Show Voice Message Button'),
+                    SettingType.CONFIG,
+                    'show_voice_message_button'),
+
             Setting(SettingKind.SWITCH,
                     _('Send Messages with Control+Enter'),
                     SettingType.CONFIG,
@@ -409,13 +384,21 @@ class GroupChats(PreferenceBox):
 class FilePreview(PreferenceBox):
     def __init__(self, *args: Any) -> None:
         sizes = {
+            0: _('No automatic preview'),
             262144: '256 KiB',
             524288: '512 KiB',
             1048576: '1 MiB',
             5242880: '5 MiB',
             10485760: '10 MiB',
+            26214400: '25 MiB',
         }
 
+        preview_actions = {}
+        for action, data in PREVIEW_ACTIONS.items():
+            if action == 'download':
+                continue
+            preview_actions[action] = data[0]
+
         settings = [
             Setting(SettingKind.SWITCH,
                     _('File Preview'),
@@ -426,15 +409,15 @@ class FilePreview(PreferenceBox):
                     _('Preview Size'),
                     SettingType.CONFIG,
                     'preview_size',
-                    desc=_('Size of preview image'),
+                    desc=_('Size of preview images in pixels'),
                     bind='enable_file_preview',
-                    props={'range_': (100, 1000)}),
+                    props={'range_': (100, 1000, 1)}),
 
             Setting(SettingKind.POPOVER,
-                    _('Allowed File Size'),
+                    _('File Size Limit'),
                     SettingType.CONFIG,
                     'preview_max_file_size',
-                    desc=_('Maximum file size for preview generation'),
+                    desc=_('Maximum file size for preview downloads'),
                     bind='enable_file_preview',
                     props={'entries': sizes}),
 
@@ -442,7 +425,7 @@ class FilePreview(PreferenceBox):
                     _('Preview in Public Group Chats'),
                     SettingType.CONFIG,
                     'preview_anonymous_muc',
-                    desc=_('Generate preview automatically in public '
+                    desc=_('Show previews automatically in public '
                            'group chats (may disclose your data)'),
                     bind='enable_file_preview'),
 
@@ -450,7 +433,7 @@ class FilePreview(PreferenceBox):
                     _('Preview all Image URLs'),
                     SettingType.CONFIG,
                     'preview_allow_all_images',
-                    desc=_('Generate preview for any URLs containing images '
+                    desc=_('Show previews for any URLs containing images '
                            '(may be unsafe)'),
                     bind='enable_file_preview'),
 
@@ -458,15 +441,16 @@ class FilePreview(PreferenceBox):
                     _('Left Click Action'),
                     SettingType.CONFIG,
                     'preview_leftclick_action',
-                    desc=_('Action when left-clicking a preview'),
+                    desc=_('Action for left-clicking a preview'),
                     bind='enable_file_preview',
-                    props={'entries': PREVIEW_CLICK_ACTIONS}),
+                    props={'entries': preview_actions}),
 
             Setting(SettingKind.SWITCH,
                     _('HTTPS Verification'),
                     SettingType.CONFIG,
                     'preview_verify_https',
-                    desc=_('Whether to check for a valid certificate'),
+                    desc=_('Whether to check for a valid certificate before '
+                           'downloading (not safe to disable)'),
                     bind='enable_file_preview'),
         ]
 
@@ -597,7 +581,7 @@ class AutoAwayDialog(SettingsDialog):
                     SettingType.CONFIG,
                     'autoawaytime',
                     desc=_('Minutes until your status gets changed'),
-                    props={'range_': (1, 720)},
+                    props={'range_': (1, 720, 1)},
                     bind='autoaway'),
 
             Setting(SettingKind.ENTRY,
@@ -625,7 +609,7 @@ class AutoExtendedAwayDialog(SettingsDialog):
                     SettingType.CONFIG,
                     'autoxatime',
                     desc=_('Minutes until your status gets changed'),
-                    props={'range_': (1, 720)},
+                    props={'range_': (1, 720, 1)},
                     bind='autoxa'),
 
             Setting(SettingKind.ENTRY,
@@ -651,6 +635,13 @@ class Themes(PreferenceBox):
         }
 
         settings = [
+            Setting(SettingKind.SPIN,
+                    _('User Interface Font Size'),
+                    SettingType.CONFIG,
+                    'app_font_size',
+                    props={'range_': (1.0, 1.5, 0.125)},
+                    callback=self._on_app_font_size_changed),
+
             Setting(SettingKind.POPOVER,
                     _('Dark Theme'),
                     SettingType.CONFIG,
@@ -671,11 +662,14 @@ class Themes(PreferenceBox):
 
         PreferenceBox.__init__(self, settings)
 
+    @staticmethod
+    def _on_app_font_size_changed(_value: float, *args: Any) -> None:
+        app.css_config.apply_app_font_size()
+
     @staticmethod
     def _get_theme_items() -> list[str]:
         theme_items = ['default']
-        for settings_theme in app.css_config.themes:
-            theme_items.append(settings_theme)
+        theme_items.extend(app.css_config.themes)
         return theme_items
 
     def update_theme_list(self) -> None:
@@ -697,42 +691,6 @@ class Themes(PreferenceBox):
         app.ged.raise_event(StyleChanged())
 
 
-class Emoji(PreferenceBox):
-    def __init__(self, *args: Any) -> None:
-        if sys.platform != 'darwin':
-            # TODO: Remove if colored emoji rendering works well on
-            # Windows and MacOS
-            PreferenceBox.__init__(self, [])
-            return
-
-        emoji_themes_items: list[str] = []
-        for theme in helpers.get_available_emoticon_themes():
-            emoji_themes_items.append(theme)
-
-        settings = [
-            Setting(SettingKind.POPOVER,
-                    _('Emoji Theme'),
-                    SettingType.CONFIG,
-                    'emoticons_theme',
-                    desc=_('Choose from various emoji styles'),
-                    props={'entries': emoji_themes_items},
-                    callback=self._on_emoticons_theme)
-        ]
-
-        PreferenceBox.__init__(self, settings)
-
-    def _on_emoticons_theme(self, *args: Any) -> None:
-        emoji_chooser.load()
-        self._toggle_emoticons()
-
-    @staticmethod
-    def _toggle_emoticons() -> None:
-        # TODO
-        # for ctrl in app.window.get_controls():
-        #     ctrl.toggle_emoticons()
-        pass
-
-
 class Server(PreferenceBox):
     def __init__(self, *args: Any) -> None:
 
@@ -773,7 +731,7 @@ class StunServerDialog(SettingsDialog):
 class Audio(PreferenceBox):
     def __init__(self, *args: Any) -> None:
 
-        deps_installed = app.is_installed('AV')
+        deps_installed = app.is_installed('GST')
 
         audio_input_devices = {}
         audio_output_devices = {}
@@ -932,6 +890,14 @@ class Miscellaneous(PreferenceBox):
                     desc=_('Use your system’s keyring to store passwords')),
         ]
 
+        if (sys.platform not in ('win32', 'darwin') and
+                package_version('keyring>=23.8.1')):
+            settings.append(
+                Setting(SettingKind.SWITCH,
+                        _('Enable KeepassXC Integration'),
+                        SettingType.CONFIG,
+                        'enable_keepassxc_integration'))
+
         if sys.platform in ('win32', 'darwin'):
             settings.append(
                 Setting(SettingKind.SWITCH,
@@ -951,7 +917,9 @@ class Miscellaneous(PreferenceBox):
 
     @staticmethod
     def _get_proxies() -> dict[str, str]:
-        return {proxy: proxy for proxy in app.settings.get_proxies()}
+        proxies = {proxy: proxy for proxy in app.settings.get_proxies()}
+        proxies['no-proxy'] = _('No Proxy')
+        return proxies
 
     @staticmethod
     def _on_proxy_edit(*args: Any) -> None:
@@ -962,10 +930,7 @@ class Miscellaneous(PreferenceBox):
         popover_row.update_entries(self._get_proxies())
 
     def _check_hints_reset(self) -> bool:
-        for hint in self._hints_list:
-            if app.settings.get(hint) is False:
-                return True
-        return False
+        return any(app.settings.get(hint) is False for hint in self._hints_list)
 
     def _on_reset_hints(self, button: Gtk.Button) -> None:
         for hint in self._hints_list:
@@ -1024,7 +989,7 @@ class Advanced(PreferenceBox):
 
     @staticmethod
     def _on_open_debug_logs(*args: Any) -> None:
-        open_file(configpaths.get('DEBUG'))
+        open_directory(configpaths.get('DEBUG'))
 
     @staticmethod
     def _on_advanced_config_editor(*args: Any) -> None:
